"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

exports.process = process;

var _reactNative = require("react-native");

var _mediaquery = require("./mediaquery.js");

var _mediaquery2 = _interopRequireDefault(_mediaquery);

var _deepmerge = require("deepmerge");

var _deepmerge2 = _interopRequireDefault(_deepmerge);

var _microMemoize = require("micro-memoize");

var _microMemoize2 = _interopRequireDefault(_microMemoize);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var PREFIX = "@media";

function isObject(obj) {
  return (typeof obj === "undefined" ? "undefined" : _typeof(obj)) === "object" && obj !== null;
}

function isMediaQuery(str) {
  return typeof str === "string" && str.indexOf(PREFIX) === 0;
}

function omit(obj, omitKey) {
  return Object.keys(obj).reduce(function (result, key) {
    if (key !== omitKey) {
      result[key] = obj[key];
    }
    return result;
  }, {});
}

function filterMq(obj) {
  return Object.keys(obj).filter(function (key) {
    return isMediaQuery(key);
  });
}

function filterNonMq(obj) {
  return Object.keys(obj).filter(function (key) {
    return !isMediaQuery(key);
  }).reduce(function (out, key) {
    out[key] = obj[key];
    return out;
  }, {});
}

var mFilterMq = (0, _microMemoize2.default)(filterMq);
var mFilterNonMq = (0, _microMemoize2.default)(filterNonMq);
var mOmit = (0, _microMemoize2.default)(omit);

function process(obj) {
  var mqKeys = mFilterMq(obj);
  var res = mFilterNonMq(obj);

  if (!mqKeys.length) {
    return res;
  }

  var matchObject = getMatchObject();

  mqKeys.forEach(function (key) {
    if (/^@media\s+(ios|android)/i.test(key)) {
      matchObject.type = _reactNative.Platform.OS;
    } else {
      matchObject.type = "screen";
    }

    var isMatch = _mediaquery2.default.match(obj.__mediaQueries[key], matchObject);
    if (isMatch) {
      res = (0, _deepmerge2.default)(res, obj[key]);
    }
  });

  return mOmit(res, "__mediaQueries");
}

function getMatchObject() {
  var win = _reactNative.Dimensions.get("window");
  return {
    width: win.width,
    height: win.height,
    orientation: win.width > win.height ? "landscape" : "portrait",
    "aspect-ratio": win.width / win.height,
    type: "screen"
  };
}